from binascii import hexlify
import hmac
from struct import pack
from typing import Callable


class RFC6979:
    """Generate a nonce per RFC6979.

    In order to avoid reusing a nonce with the same key when signing two different messages (which
    leaks the private key) RFC6979 provides a deterministic method for generating nonces. This is
    based on using a pseudo-random function (HMAC) to derive a nonce from the message and private
    key. More info here: http://tools.ietf.org/html/rfc6979.

    Attributes:
        |  msg (bytes): A message being signed.
        |  x (int): An ECDSA private key.
        |  q (int): The order of the generator point of the curve being used to sign the message.
        |  hashfunc (_hashlib.HASH): The hash function used to compress the message.
        |  prehashed (bool): Whether the signature is on a pre-hashed message.
    """
    def __init__(self, msg: bytes, x: int, q: int, hashfunc: Callable, prehashed: bool = False):
        self.x = x
        self.q = q
        self.msg = msg_bytes(msg)
        self.qlen = len(bin(q)) - 2  # -2 for the leading '0b'
        self.rlen = ((self.qlen + 7) // 8) * 8
        self.hashfunc = hashfunc
        self.prehashed = prehashed

    def _bits2int(self, b: bytes) -> int:
        """ http://tools.ietf.org/html/rfc6979#section-2.3.2 """
        i = int(hexlify(b), 16)
        blen = len(b) * 8

        if blen > self.qlen:
            i >>= (blen - self.qlen)

        return i

    def _int2octets(self, x: int) -> bytes:
        """ http://tools.ietf.org/html/rfc6979#section-2.3.3 """
        octets = b''

        while x > 0:
            octets = pack('=B', (0xff & x)) + octets
            x >>= 8

        padding = b'\x00' * ((self.rlen // 8) - len(octets))
        return padding + octets

    def _bits2octets(self, b: bytes) -> bytes:
        """ http://tools.ietf.org/html/rfc6979#section-2.3.4 """
        z1 = self._bits2int(b)  # -2 for the leading '0b'
        z2 = z1 % self.q
        return self._int2octets(z2)

    def gen_nonce(self):
        """ http://tools.ietf.org/html/rfc6979#section-3.2 """
        hash_size = self.hashfunc().digest_size
        if self.prehashed:
            h1 = self.msg
        else:
            h1 = self.hashfunc(self.msg).digest()
        key_and_msg = self._int2octets(self.x) + self._bits2octets(h1)

        v = b''.join([b'\x01' for _ in range(hash_size)])
        k = b''.join([b'\x00' for _ in range(hash_size)])

        k = hmac.new(k, v + b'\x00' + key_and_msg, self.hashfunc).digest()
        v = hmac.new(k, v, self.hashfunc).digest()
        k = hmac.new(k, v + b'\x01' + key_and_msg, self.hashfunc).digest()
        v = hmac.new(k, v, self.hashfunc).digest()

        while True:
            t = b''

            while len(t) * 8 < self.qlen:
                v = hmac.new(k, v, self.hashfunc).digest()
                t = t + v

            nonce = self._bits2int(t)
            if nonce >= 1 and nonce < self.q:
                return nonce

            k = hmac.new(k, v + b'\x00', self.hashfunc).digest()
            v = hmac.new(k, v, self.hashfunc).digest()


def _tonelli_shanks(n: int, p: int) -> (int, int):
    """A generic algorithm for computng modular square roots."""
    Q, S = p - 1, 0
    while Q % 2 == 0:
        Q, S = Q // 2, S + 1

    z = 2
    while pow(z, (p - 1) // 2, p) != (-1 % p):
        z += 1

    M, c, t, R = S, pow(z, Q, p), pow(n, Q, p), pow(n, (Q + 1) // 2, p)
    while t != 1:
        for i in range(1, M):
            if pow(t, 2**i, p) == 1:
                break

        b = pow(c, 2**(M - i - 1), p)
        M, c, t, R = i, pow(b, 2, p), (t * b * b) % p, (R * b) % p

    return R, -R % p


def mod_sqrt(a: int, p: int) -> (int, int):
    """Compute the square root of :math:`a \pmod{p}`

    In other words, find a value :math:`x` such that :math:`x^2 \equiv a \pmod{p}`.

    Args:
        |  a (int): The value whose root to take.
        |  p (int): The prime whose field to perform the square root in.

    Returns:
        (int, int): the two values of :math:`x` satisfying :math:`x^2 \equiv a \pmod{p}`.
    """
    if p % 4 == 3:
        k = (p - 3) // 4
        x = pow(a, k + 1, p)
        return x, (-x % p)
    else:
        return _tonelli_shanks(a, p)


def msg_bytes(msg) -> bytes:
    """Return bytes in a consistent way for a given message.

    The message is expected to be either a string, bytes, or an array of bytes.

    Args:
        |  msg (str|bytes|bytearray): The data to transform.

    Returns:
        bytes: The byte encoded data.

    Raises:
        ValueError: If the data cannot be encoded as bytes.
    """
    if isinstance(msg, bytes):
        return msg
    elif isinstance(msg, str):
        return msg.encode()
    elif isinstance(msg, bytearray):
        return bytes(msg)
    else:
        raise ValueError('Msg "{}" of type {} cannot be converted to bytes'.format(
            msg, type(msg)))


def validate_type(instance: object, expected_type: type):
    """Validate that instance is an instance of the expected_type.

    Args:
        |  instance: The object whose type is being checked
        |  expected_type: The expected type of instance
        |  var_name: The name of the object

    Raises:
         TypeError: If instance is not of type expected_type
    """
    if not isinstance(instance, expected_type):
        raise TypeError('Expected a value of type {}, got a value of type {}'.format(
            expected_type, type(instance)))



from ..curve import (
    P192, P224, P256, P384, P521, secp192k1, secp224k1, secp256k1, brainpoolP160r1, brainpoolP192r1,
    brainpoolP224r1, brainpoolP256r1, brainpoolP320r1, brainpoolP384r1, brainpoolP512r1
)

CURVES = [
    P192, P224, P256, P384, P521, secp192k1, secp224k1, secp256k1, brainpoolP160r1, brainpoolP192r1,
    brainpoolP224r1, brainpoolP256r1, brainpoolP320r1, brainpoolP384r1, brainpoolP512r1
]



from binascii import hexlify
from hashlib import sha256
from os import urandom
from typing import Optional, Tuple

from .curve import Curve, P256
from .ecdsa import verify
from .encoding.pem import PEMEncoder
from .point import Point
from .util import mod_sqrt, msg_bytes


def gen_keypair(curve: Curve) -> Tuple[int, Point]:
    """Generate a keypair that consists of a private key and a public key.

    The private key :math:`d` is an integer generated via a cryptographically secure random number
    generator that lies in the range :math:`[1,n)`, where :math:`n` is the curve order. The public
    key :math:`Q` is a point on the curve calculated as :math:`Q = dG`, where :math:`G` is the
    curve's base point.

    Args:
        curve (fastecdsa.curve.Curve): The curve over which the keypair will be calulated.

    Returns:
        int, fastecdsa.point.Point: Returns a tuple with the private key first and public key
        second.
    """
    private_key = gen_private_key(curve)
    public_key = get_public_key(private_key, curve)
    return private_key, public_key


def gen_private_key(curve: Curve, randfunc=urandom) -> int:
    """Generate a private key to sign data with.

    The private key :math:`d` is an integer generated via a cryptographically secure random number
    generator that lies in the range :math:`[1,n)`, where :math:`n` is the curve order. The default
    random number generator used is /dev/urandom.

    Args:
        |  curve (fastecdsa.curve.Curve): The curve over which the key will be calulated.
        |  randfunc (function): A function taking one argument 'n' and returning a bytestring
                                of n random bytes suitable for cryptographic use.
                                The default is "os.urandom"
    Returns:
        int: Returns a positive integer smaller than the curve order.
    """
    order_bits = 0
    order = curve.q

    while order > 0:
        order >>= 1
        order_bits += 1

    order_bytes = (order_bits + 7) // 8  # randfunc only takes bytes
    extra_bits = order_bytes * 8 - order_bits  # bits to shave off after getting bytes

    rand = int(hexlify(randfunc(order_bytes)), 16)
    rand >>= extra_bits

    # no modding by group order or we'll introduce biases
    while rand >= curve.q:
        rand = int(hexlify(randfunc(order_bytes)), 16)
        rand >>= extra_bits

    return rand


def get_public_key(d: int, curve: Curve) -> Point:
    """Generate a public key from a private key.

    The public key :math:`Q` is a point on the curve calculated as :math:`Q = dG`, where :math:`d`
    is the private key and :math:`G` is the curve's base point.

    Args:
        |  d (long): An integer representing the private key.
        |  curve (fastecdsa.curve.Curve): The curve over which the key will be calulated.

    Returns:
        fastecdsa.point.Point: The public key, a point on the given curve.
    """
    return d * curve.G


def get_public_keys_from_sig(sig: (int, int), msg, curve: Curve = P256, hashfunc=sha256) -> Tuple[Point, Point]:
    """Recover the public keys that can verify a signature / message pair.

    Args:
        |  sig (int, int): A ECDSA signature.
        |  msg (str|bytes|bytearray): The message corresponding to the signature.
        |  curve (fastecdsa.curve.Curve): The curve used to sign the message.
        |  hashfunc (_hashlib.HASH): The hash function used to compress the message.

    Returns:
        (fastecdsa.point.Point, fastecdsa.point.Point): The public keys that can verify the
                                                        signature for the message.
    """
    r, s = sig
    rinv = pow(r, curve.q - 2, curve.q)

    z = int(hashfunc(msg_bytes(msg)).hexdigest(), 16)
    hash_bit_length = hashfunc().digest_size * 8
    if curve.q.bit_length() < hash_bit_length:
        z >>= (hash_bit_length - curve.q.bit_length())

    y_squared = (r * r * r + curve.a * r + curve.b) % curve.p
    y1, y2 = mod_sqrt(y_squared, curve.p)
    R1, R2 = Point(r, y1, curve=curve), Point(r, y2, curve=curve)

    Qs = rinv * (s * R1 - z * curve.G), rinv * (s * R2 - z * curve.G)
    for Q in Qs:
        if not verify(sig, msg, Q, curve=curve, hashfunc=hashfunc):
            raise ValueError('Could not recover public key, is the signature ({}) a valid '
                             'signature for the message ({}) over the given curve ({}) using the '
                             'given hash function ({})?'.format(sig, msg, curve, hashfunc))
    return Qs


def export_key(key, curve: Curve = None, filepath: str = None, encoder=PEMEncoder):
    """Export a public or private EC key in PEM format.

    Args:
        |   key (fastecdsa.point.Point | int): A public or private EC key
        |   curve (fastecdsa.curve.Curve): The curve corresponding to the key (required if the
            key is a private key)
        |   filepath (str): Where to save the exported key. If None the key is simply printed.
        |   encoder (type): The class used to encode the key
    """
    # encode a public key
    if isinstance(key, Point):
        encoded = encoder.encode_public_key(key)

    # throw error for ambiguous private keys
    elif curve is None:
        raise ValueError('curve parameter cannot be \'None\' when exporting a private key')

    # encode a private key
    else:
        pubkey = key * curve.G
        encoded = encoder.encode_private_key(key, Q=pubkey)

    # return binary data or write to file
    if filepath is None:
        return encoded
    else:
        # some encoder output strings, others bytes, need to determine what mode to write in
        write_mode = 'w' + ('b' if getattr(encoder, 'binary_data', False) else '')
        with open(filepath, write_mode) as f:
            f.write(encoded)


def import_key(filepath: str, curve: Curve = None, public: bool = False, decoder=PEMEncoder
               ) -> Tuple[Optional[int], Point]:
    """Import a public or private EC key in PEM format.

    Args:
        |  filepath (str): The location of the key file
        |  public (bool): Indicates if the key file is a public key
        |  decoder (fastecdsa.encoding.KeyEncoder): The class used to parse the key

    Returns:
        (long, fastecdsa.point.Point): A (private key, public key) tuple. If a public key was
        imported then the first value will be None.
    """
    # some decoders read strings, others bytes, need to determine what mode to write in
    read_mode = 'r' + ('b' if getattr(decoder, 'binary_data', False) else '')
    with open(filepath, read_mode) as f:
        data = f.read()

    if public:
        return None, decoder.decode_public_key(data, curve)
    else:
        return decoder.decode_private_key(data)



from abc import ABCMeta, abstractmethod

from ..point import Point


class KeyEncoder:
    """Base class that any encoding class for EC keys should derive from.

    All overriding methods should be static. If your key encoder writes binary
    data you must have a field named :code:`binary_data` set to :code:`True` in
    order for keys to correctly read from and write to disk.
    """
    __metaclass__ = ABCMeta

    @staticmethod
    @abstractmethod
    def encode_public_key(Q: Point):
        pass

    @staticmethod
    @abstractmethod
    def encode_private_key(d: int):
        pass

    @staticmethod
    @abstractmethod
    def decode_public_key(data) -> Point:
        pass

    @staticmethod
    @abstractmethod
    def decode_private_key(data) -> (int, Point):
        pass


class SigEncoder:
    """Base class that any encoding class for EC signatures should derive from.

    All overriding methods should be static.
    """
    __metaclass__ = ABCMeta

    @staticmethod
    @abstractmethod
    def encode_signature(r: int, s: int) -> bytes:
        pass

    @staticmethod
    @abstractmethod
    def decode_signature(binary_data: bytes) -> (int, int):
        pass



from struct import pack

from . import SigEncoder
from .asn1 import ASN1EncodingError, INTEGER, SEQUENCE, asn1_structure, parse_asn1_int, parse_asn1_length
from .util import bytes_to_int, int_to_bytes


class InvalidDerSignature(Exception):
    pass


class DEREncoder(SigEncoder):
    @staticmethod
    def encode_signature(r: int, s: int) -> bytes:
        """Encode an EC signature in serialized DER format as described in
           https://tools.ietf.org/html/rfc2459 (section 7.2.2) and as detailed by
           bip-0066

        Args:
            r, s

        Returns:
            bytes: The DER encoded signature

        """
        r_bytes = int_to_bytes(r)
        if r_bytes[0] & 0x80:
            r_bytes = b"\x00" + r_bytes
        s_bytes = int_to_bytes(s)
        if s_bytes[0] & 0x80:
            s_bytes = b"\x00" + s_bytes

        r_asn1 = asn1_structure(INTEGER, r_bytes)
        s_asn1 = asn1_structure(INTEGER, s_bytes)
        return asn1_structure(SEQUENCE, r_asn1 + s_asn1)

    @staticmethod
    def decode_signature(sig: bytes) -> (int, int):
        """Decode an EC signature from serialized DER format as described in
           https://tools.ietf.org/html/rfc2459 (section 7.2.2) and as detailed by
           bip-0066

           Returns (r,s)
        """
        def _validate_int_bytes(data: bytes):
            # check for negative values, indicated by leading 1 bit
            if data[0] & 0x80:
                raise InvalidDerSignature("Signature contains a negative value")

            # check for leading 0x00s that aren't there to disambiguate possible negative values
            if data[0] == 0x00 and not data[1] & 0x80:
                raise InvalidDerSignature("Invalid leading 0x00 byte in ASN.1 integer")

        # overarching structure must be a sequence
        if not sig or sig[0] != ord(SEQUENCE):
            raise InvalidDerSignature("First byte should be ASN.1 SEQUENCE")

        try:
            seqlen, sequence, leftover = parse_asn1_length(sig[1:])
        except ASN1EncodingError as asn1_error:
            raise InvalidDerSignature(asn1_error)

        # sequence should be entirety remaining data
        if leftover:
            raise InvalidDerSignature("Expected a sequence of {} bytes, got {}".format(
                seqlen, len(sequence + leftover)))

        try:
            rlen, r, sdata = parse_asn1_int(sequence)
            slen, s, _ = parse_asn1_int(sdata)
        except ASN1EncodingError as asn1_error:
            raise InvalidDerSignature(asn1_error)

        _validate_int_bytes(r)
        _validate_int_bytes(s)
        return bytes_to_int(r), bytes_to_int(s)




from hashlib import sha256
from unittest import TestCase

from ..curve import P256
from ..ecdsa import sign, verify
from ..keys import gen_keypair


class TestPrehashed(TestCase):
    def test_prehashed_bytes(self):
        d, Q = gen_keypair(P256)
        prehashed_message = sha256(b"").digest()

        r, s = sign(prehashed_message, d, prehashed=True)
        self.assertTrue(verify((r, s), prehashed_message, Q, prehashed=True))



from hashlib import sha1, sha224, sha256, sha384, sha512
from unittest import TestCase

from ..curve import P256
from ..ecdsa import sign, verify
from ..point import Point


class TestP256ECDSA(TestCase):
    """ case taken from http://tools.ietf.org/html/rfc6979#appendix-A.2.5 """

    def test_ecdsa_P256_SHA1_sign(self):
        d = 0xC9AFA9D845BA75166B5C215767B1D6934E50C3DB36E89B127B8A622B120F6721
        expected = (
            0x61340C88C3AAEBEB4F6D667F672CA9759A6CCAA9FA8811313039EE4A35471D32,
            0x6D7F147DAC089441BB2E2FE8F7A3FA264B9C475098FDCF6E00D7C996E1B8B7EB,
        )
        sig = sign('sample', d, curve=P256, hashfunc=sha1)
        self.assertEqual(sig, expected)

        Q = d * P256.G
        self.assertTrue(verify(sig, 'sample', Q, curve=P256, hashfunc=sha1))

    """ case taken from http://tools.ietf.org/html/rfc6979#appendix-A.2.5 """

    def test_ecdsa_P256_SHA224_sign(self):
        d = 0xC9AFA9D845BA75166B5C215767B1D6934E50C3DB36E89B127B8A622B120F6721
        expected = (
            0x53B2FFF5D1752B2C689DF257C04C40A587FABABB3F6FC2702F1343AF7CA9AA3F,
            0xB9AFB64FDC03DC1A131C7D2386D11E349F070AA432A4ACC918BEA988BF75C74C
        )
        sig = sign('sample', d, curve=P256, hashfunc=sha224)
        self.assertEqual(sig, expected)

        Q = d * P256.G
        self.assertTrue(verify(sig, 'sample', Q, curve=P256, hashfunc=sha224))

    """ case taken from http://tools.ietf.org/html/rfc6979#appendix-A.2.5 """

    def test_ecdsa_P256_SHA2_sign(self):
        d = 0xC9AFA9D845BA75166B5C215767B1D6934E50C3DB36E89B127B8A622B120F6721
        expected = (
            0xEFD48B2AACB6A8FD1140DD9CD45E81D69D2C877B56AAF991C34D0EA84EAF3716,
            0xF7CB1C942D657C41D436C7A1B6E29F65F3E900DBB9AFF4064DC4AB2F843ACDA8
        )
        sig = sign('sample', d, curve=P256, hashfunc=sha256)
        self.assertEqual(sig, expected)

        Q = d * P256.G
        self.assertTrue(verify(sig, 'sample', Q, curve=P256, hashfunc=sha256))

    """ case taken from http://tools.ietf.org/html/rfc6979#appendix-A.2.5 """

    def test_ecdsa_P256_SHA384_sign(self):
        d = 0xC9AFA9D845BA75166B5C215767B1D6934E50C3DB36E89B127B8A622B120F6721
        expected = (
            0x0EAFEA039B20E9B42309FB1D89E213057CBF973DC0CFC8F129EDDDC800EF7719,
            0x4861F0491E6998B9455193E34E7B0D284DDD7149A74B95B9261F13ABDE940954
        )
        sig = sign('sample', d, curve=P256, hashfunc=sha384)
        self.assertEqual(sig, expected)

        Q = d * P256.G
        self.assertTrue(verify(sig, 'sample', Q, curve=P256, hashfunc=sha384))

    """ case taken from http://tools.ietf.org/html/rfc6979#appendix-A.2.5 """

    def test_ecdsa_P256_SHA512_sign(self):
        d = 0xC9AFA9D845BA75166B5C215767B1D6934E50C3DB36E89B127B8A622B120F6721
        expected = (
            0x8496A60B5E9B47C825488827E0495B0E3FA109EC4568FD3F8D1097678EB97F00,
            0x2362AB1ADBE2B8ADF9CB9EDAB740EA6049C028114F2460F96554F61FAE3302FE
        )
        sig = sign('sample', d, curve=P256, hashfunc=sha512)
        self.assertEqual(sig, expected)

        Q = d * P256.G
        self.assertTrue(verify(sig, 'sample', Q, curve=P256, hashfunc=sha512))

    """ case taken from https://www.nsa.gov/ia/_files/ecdsa.pdf """

    def test_ecdsa_P256_verify(self):
        Q = Point(
            0x8101ece47464a6ead70cf69a6e2bd3d88691a3262d22cba4f7635eaff26680a8,
            0xd8a12ba61d599235f67d9cb4d58f1783d3ca43e78f0a5abaa624079936c0c3a9,
            curve=P256
        )
        msg = 'This is only a test message. It is 48 bytes long'
        sig = (
            0x7214bc9647160bbd39ff2f80533f5dc6ddd70ddf86bb815661e805d5d4e6f27c,
            0x7d1ff961980f961bdaa3233b6209f4013317d3e3f9e1493592dbeaa1af2bc367
        )
        self.assertTrue(verify(sig, msg, Q, curve=P256, hashfunc=sha256))

        sig = (
            0x7214bc9647160bbd39ff2f80533f5dc6ddd70ddf86bb815661e805d5d4e6fbad,
            0x7d1ff961980f961bdaa3233b6209f4013317d3e3f9e1493592dbeaa1af2bc367
        )
        self.assertFalse(verify(sig, msg, Q, curve=P256, hashfunc=sha256))


from hashlib import sha1, sha224, sha256, sha384, sha512
from re import findall, DOTALL
from urllib.request import urlopen
from unittest import TestCase

from ..curve import P192, P224, P256, P384, P521
from ..ecdsa import sign
from ..util import RFC6979


class TestRFC6979ECDSA(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.rfc6979_text = urlopen('https://tools.ietf.org/rfc/rfc6979.txt').read().decode()
        cls.hash_lookup = {
            '1': sha1,
            '224': sha224,
            '256': sha256,
            '384': sha384,
            '512': sha512
        }

    def test_p192_rfc6979_ecdsa(self):
        curve_tests = findall(r'curve: NIST P-192(.*)curve: NIST P-224', self.rfc6979_text, flags=DOTALL)[0]

        q = int(findall(r'q = ([0-9A-F]*)', curve_tests)[0], 16)
        x = int(findall(r'x = ([0-9A-F]*)', curve_tests)[0], 16)

        test_regex = r'With SHA-(\d+), message = "([a-zA-Z]*)":\n' \
                     r'\s*k = ([0-9A-F]*)\n' \
                     r'\s*r = ([0-9A-F]*)\n' \
                     r'\s*s = ([0-9A-F]*)\n'

        for test in findall(test_regex, curve_tests):
            h = self.hash_lookup[test[0]]
            msg = test[1]
            k = int(test[2], 16)
            r = int(test[3], 16)
            s = int(test[4], 16)

            self.assertEqual(k, RFC6979(msg, x, q, h).gen_nonce())
            self.assertEqual((r, s), sign(msg, x, curve=P192, hashfunc=h))

    def test_p224_rfc6979_ecdsa(self):
        curve_tests = findall(r'curve: NIST P-224(.*)curve: NIST P-256', self.rfc6979_text, flags=DOTALL)[0]

        q = int(findall(r'q = ([0-9A-F]*)', curve_tests)[0], 16)
        x = int(findall(r'x = ([0-9A-F]*)', curve_tests)[0], 16)

        test_regex = r'With SHA-(\d+), message = "([a-zA-Z]*)":\n' \
                     r'\s*k = ([0-9A-F]*)\n' \
                     r'\s*r = ([0-9A-F]*)\n' \
                     r'\s*s = ([0-9A-F]*)\n'

        for test in findall(test_regex, curve_tests):
            h = self.hash_lookup[test[0]]
            msg = test[1]
            k = int(test[2], 16)
            r = int(test[3], 16)
            s = int(test[4], 16)

            self.assertEqual(k, RFC6979(msg, x, q, h).gen_nonce())
            self.assertEqual((r, s), sign(msg, x, curve=P224, hashfunc=h))

    def test_p256_rfc6979_ecdsa(self):
        curve_tests = findall(r'curve: NIST P-256(.*)curve: NIST P-384', self.rfc6979_text, flags=DOTALL)[0]

        q = int(findall(r'q = ([0-9A-F]*)', curve_tests)[0], 16)
        x = int(findall(r'x = ([0-9A-F]*)', curve_tests)[0], 16)

        test_regex = r'With SHA-(\d+), message = "([a-zA-Z]*)":\n' \
                     r'\s*k = ([0-9A-F]*)\n' \
                     r'\s*r = ([0-9A-F]*)\n' \
                     r'\s*s = ([0-9A-F]*)\n'

        for test in findall(test_regex, curve_tests):
            h = self.hash_lookup[test[0]]
            msg = test[1]
            k = int(test[2], 16)
            r = int(test[3], 16)
            s = int(test[4], 16)

            self.assertEqual(k, RFC6979(msg, x, q, h).gen_nonce())
            self.assertEqual((r, s), sign(msg, x, curve=P256, hashfunc=h))

    def test_p384_rfc6979_ecdsa(self):
        curve_tests = findall(r'curve: NIST P-384(.*)curve: NIST P-521', self.rfc6979_text, flags=DOTALL)[0]

        q_parts = findall(r'q = ([0-9A-F]*)\n\s*([0-9A-F]*)', curve_tests)[0]
        q = int(q_parts[0] + q_parts[1], 16)
        x_parts = findall(r'x = ([0-9A-F]*)\n\s*([0-9A-F]*)', curve_tests)[0]
        x = int(x_parts[0] + x_parts[1], 16)

        test_regex = r'With SHA-(\d+), message = "([a-zA-Z]*)":\n' \
                     r'\s*k = ([0-9A-F]*)\n\s*([0-9A-F]*)\n' \
                     r'\s*r = ([0-9A-F]*)\n\s*([0-9A-F]*)\n' \
                     r'\s*s = ([0-9A-F]*)\n\s*([0-9A-F]*)\n'

        for test in findall(test_regex, curve_tests):
            h = self.hash_lookup[test[0]]
            msg = test[1]
            k = int(test[2] + test[3], 16)
            r = int(test[4] + test[5], 16)
            s = int(test[6] + test[7], 16)

            self.assertEqual(k, RFC6979(msg, x, q, h).gen_nonce())
            self.assertEqual((r, s), sign(msg, x, curve=P384, hashfunc=h))

    def test_p521_rfc6979_ecdsa(self):
        curve_tests = findall(r'curve: NIST P-521(.*)curve: NIST K-163', self.rfc6979_text, flags=DOTALL)[0]

        q_parts = findall(r'q = ([0-9A-F]*)\n\s*([0-9A-F]*)\n\s*([0-9A-F]*)', curve_tests)[0]
        q = int(q_parts[0] + q_parts[1] + q_parts[2], 16)
        x_parts = findall(r'x = ([0-9A-F]*)\n\s*([0-9A-F]*)\n\s*([0-9A-F]*)', curve_tests)[0]
        x = int(x_parts[0] + x_parts[1] + x_parts[2], 16)

        test_regex = r'With SHA-(\d+), message = "([a-zA-Z]*)":\n' \
                     r'\s*k = ([0-9A-F]*)\n\s*([0-9A-F]*)\n\s*([0-9A-F]*)\n' \
                     r'\s*r = ([0-9A-F]*)\n\s*([0-9A-F]*)\n\s*([0-9A-F]*)\n' \
                     r'\s*s = ([0-9A-F]*)\n\s*([0-9A-F]*)\n\s*([0-9A-F]*)\n'

        for test in findall(test_regex, curve_tests):
            h = self.hash_lookup[test[0]]
            msg = test[1]
            k = int(test[2] + test[3] + test[4], 16)
            r = int(test[5] + test[6] + test[7], 16)
            s = int(test[8] + test[9] + test[10], 16)

            self.assertEqual(k, RFC6979(msg, x, q, h).gen_nonce())
            self.assertEqual((r, s), sign(msg, x, curve=P521, hashfunc=h))



from os import remove
from unittest import TestCase

from . import CURVES
from ..encoding.pem import PEMEncoder
from ..encoding.sec1 import SEC1Encoder
from ..keys import export_key, gen_keypair, import_key

TEST_FILE_PATH = 'fastecdsa_test_key.pem'


class TestExportImport(TestCase):
    def test_export_import_private_key(self):
        for curve in CURVES:
            d, Q = gen_keypair(curve)
            export_key(d, curve=curve, filepath=TEST_FILE_PATH)
            d_, Q_ = import_key(filepath=TEST_FILE_PATH, curve=curve)

            self.assertEqual(d, d_)
            self.assertEqual(Q, Q_)

            remove(TEST_FILE_PATH)

    def test_export_import_public_key(self):
        for curve in CURVES:
            for encoder in (PEMEncoder, SEC1Encoder):
                d, Q = gen_keypair(curve)
                export_key(Q, curve=curve, filepath=TEST_FILE_PATH, encoder=encoder)
                d_, Q_ = import_key(filepath=TEST_FILE_PATH, curve=curve, public=True, decoder=encoder)

                self.assertIsNone(d_)
                self.assertEqual(Q, Q_)

                remove(TEST_FILE_PATH)



from hashlib import sha256
from unittest import TestCase

from . import CURVES
from ..ecdsa import sign
from ..keys import gen_keypair, get_public_keys_from_sig


class TestKeyRecovery(TestCase):
    def test_key_recovery(self):
        for curve in CURVES:
            d, Q = gen_keypair(curve)
            msg = 'https://crypto.stackexchange.com/questions/18105/how-does-recovering-the-' \
                  'public-key-from-an-ecdsa-signature-work'
            sig = sign(msg, d, curve=curve)

            Qs = get_public_keys_from_sig(sig, msg, curve=curve, hashfunc=sha256)
            self.assertTrue(Q in Qs)


from setuptools import setup, Extension, Command
from sys import executable

class BenchmarkCommand(Command):
    user_options = []
    description = "Benchmark this package"

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        from subprocess import call, PIPE
        call([executable, 'setup.py', 'build_ext', '--inplace'], stdout=PIPE)
        call([executable, '-m', 'fastecdsa.benchmark'])


curvemath = Extension(
    'fastecdsa.curvemath',
    include_dirs=['src/'],
    libraries=['gmp'],
    sources=['src/curveMath.c', 'src/curve.c', 'src/point.c'],
    extra_compile_args=['-O2']
)

_ecdsa = Extension(
    'fastecdsa._ecdsa',
    include_dirs=['src/'],
    libraries=['gmp'],
    sources=['src/_ecdsa.c', 'src/curveMath.c', 'src/curve.c', 'src/point.c'],
    extra_compile_args=['-O2']
)

setup(
    cmdclass={'benchmark': BenchmarkCommand},
    ext_modules=[curvemath, _ecdsa],
)



from struct import pack


def int_bytelen(x: int) -> int:
    length = 0

    while x:
        length += 1
        x >>= 8

    return length


def int_to_bytes(x: int) -> bytes:
    bs = b''

    while x:
        bs = pack('=B', (0xff & x)) + bs
        x >>= 8

    return bs


def bytes_to_int(bytestr: bytes) -> int:
    """Make an integer from a big endian bytestring."""
    value = 0
    for byte_intval in bytestr:
        value = value * 256 + byte_intval
    return value



from binascii import a2b_base64, b2a_base64, hexlify
from textwrap import wrap

from . import KeyEncoder
from .asn1 import (
    BIT_STRING, OBJECT_IDENTIFIER, OCTET_STRING, PARAMETERS, PUBLIC_KEY, SEQUENCE,
    asn1_ecpublickey, asn1_ecversion, asn1_oid, asn1_private_key, asn1_public_key, asn1_structure, parse_asn1_length
)
from ..curve import Curve
from ..point import Point

EC_PRIVATE_HEADER = '-----BEGIN EC PRIVATE KEY-----'
EC_PRIVATE_FOOTER = '-----END EC PRIVATE KEY-----'

EC_PUBLIC_HEADER = '-----BEGIN PUBLIC KEY-----'
EC_PUBLIC_FOOTER = '-----END PUBLIC KEY-----'


class PEMEncoder(KeyEncoder):
    ASN1_PARSED_DATA = []
    binary_data = False

    @staticmethod
    def _parse_ascii_armored_base64(data: str) -> bytes:
        """Convert an ASCII armored key to raw binary data"""
        data = data.strip()
        lines = (line for line in data.split('\n'))
        header = next(lines).rstrip()

        base64_data = ''
        line = next(lines).rstrip()

        while line and (line != EC_PRIVATE_FOOTER) and (line != EC_PUBLIC_FOOTER):
            base64_data += line
            line = next(lines).rstrip()

        return a2b_base64(base64_data)

    @staticmethod
    def _parse_asn1_structure(data: bytes):
        """Recursively parse ASN.1 data"""
        data_type = data[:1]
        length, data, remaining = parse_asn1_length(data[1:])

        if data_type in [OCTET_STRING, BIT_STRING, OBJECT_IDENTIFIER]:
            PEMEncoder.ASN1_PARSED_DATA.append((data_type, data))
        elif data_type in [SEQUENCE, PUBLIC_KEY, PARAMETERS]:
            PEMEncoder._parse_asn1_structure(data)

        if remaining:
            PEMEncoder._parse_asn1_structure(remaining)

    @staticmethod
    def encode_public_key(Q: Point) -> str:
        """Encode an EC public key as described in `RFC 5480 <https://tools.ietf.org/html/rfc5480>`_.

        Returns:
            str: The ASCII armored encoded EC public key.
        """
        algorithm = asn1_ecpublickey()
        oid = asn1_oid(Q.curve)
        parameters = asn1_structure(SEQUENCE, algorithm + oid)
        public_key = asn1_public_key(Q)

        sequence = parameters + public_key
        ec_public_key = asn1_structure(SEQUENCE, sequence)
        b64_data = '\n'.join(wrap(b2a_base64(ec_public_key).decode(), 64))

        return EC_PUBLIC_HEADER + '\n' + b64_data + '\n' + EC_PUBLIC_FOOTER

    @staticmethod
    def encode_private_key(d: int, Q: Point = None, curve: Curve = None) -> str:
        """Encode an EC keypair as described in `RFC 5915 <https://tools.ietf.org/html/rfc5915.html>`_.

        Args:
            | d (long): An ECDSA private key.
            | Q (fastecdsa.point.Point): The ECDSA public key.
            | curve (fastecdsa.curve.Curve): The curve that the private key is for.

        Returns:
            str: The ASCII armored encoded EC keypair.
        """
        if Q is None and curve is None:
            raise ValueError('Ambiguous encoding, public key or curve must passed as an argument')
        elif Q is None:
            Q = d * curve.G

        version = asn1_ecversion()
        private_key = asn1_private_key(d, Q.curve)
        oid = asn1_oid(Q.curve)
        parameters = asn1_structure(PARAMETERS, oid)
        public_key_bitstring = asn1_public_key(Q)
        public_key = asn1_structure(PUBLIC_KEY, public_key_bitstring)

        sequence = version + private_key + parameters + public_key
        ec_private_key = asn1_structure(SEQUENCE, sequence)
        b64_data = '\n'.join(wrap(b2a_base64(ec_private_key).decode(), 64))

        return EC_PRIVATE_HEADER + '\n' + b64_data + '\n' + EC_PRIVATE_FOOTER

    @staticmethod
    def decode_public_key(pemdata: str, curve: Curve = None) -> Point:
        """Delegate to private key decoding but return only the public key"""
        _, Q = PEMEncoder.decode_private_key(pemdata)
        return Q

    @staticmethod
    def decode_private_key(pemdata: str) -> (int, Point):
        """Decode an EC key as described in `RFC 5915 <https://tools.ietf.org/html/rfc5915.html>`_ and
        `RFC 5480 <https://tools.ietf.org/html/rfc5480>`_.

        Args:
            pemdata (bytes): A sequence of bytes representing an encoded EC key.

        Returns:
            (long, fastecdsa.point.Point): A private key, public key tuple. If the encoded key was a
            public key the first entry in the tuple is None.
        """
        pemdata = PEMEncoder._parse_ascii_armored_base64(pemdata)
        PEMEncoder._parse_asn1_structure(pemdata)

        d, x, y, curve = None, None, None, None
        for (value_type, value) in PEMEncoder.ASN1_PARSED_DATA:
            if value_type == OCTET_STRING:
                d = int(hexlify(value), 16)
            elif value_type == OBJECT_IDENTIFIER and curve is None:
                curve = Curve.get_curve_by_oid(value)
            elif value_type == BIT_STRING:
                value = value[2:]  # strip off b'\x00\x04'
                x = int(hexlify(value[:len(value) // 2]), 16)
                y = int(hexlify(value[len(value) // 2:]), 16)

        PEMEncoder.ASN1_PARSED_DATA = []
        Q = None if (x is None) or (y is None) else Point(x, y, curve)
        return d, Q
